{% extends "base.html" %}
{% block title %}Create Relationship - Personaut{% endblock %}

{% block content %}
<header class="page-header">
    <h1>Create Relationship</h1>
</header>
<form method="POST" class="form-container">
    <div class="form-group">
        <label>Type</label>
        <select name="type">
            <option>Friend</option>
            <option>Colleague</option>
            <option>Family</option>
        </select>
    </div>
    <div class="form-actions">
        <a href="{{ url_for('relationships.list_view') }}" class="btn btn-secondary">Cancel</a>
        <button class="btn btn-primary">Create</button>
    </div>
</form>
{% endblock %}