"""Views package for the web UI."""

from __future__ import annotations

from personaut.server.ui.views import api_helpers, chat, dashboard, individuals, relationships, situations


__all__ = [
    "api_helpers",
    "chat",
    "dashboard",
    "individuals",
    "relationships",
    "situations",
]
