{% extends "base.html" %}
{% block title %}Not Found - Personaut{% endblock %}

{% block content %}
<header class="page-header">
    <h1>Situation Not Found</h1>
    <p class="subtitle">ID: {{ id }}</p>
</header>
<p class="empty-state">Situation not found.</p>
<a href="{{ url_for('situations.list_view') }}" class="btn btn-secondary">← Back to list</a>
{% endblock %}