"""API routes package."""

from __future__ import annotations

from personaut.server.api.routes import (
    emotions,
    individuals,
    masks,
    memories,
    relationships,
    sessions,
    situations,
    triggers,
    websocket,
)


__all__ = [
    "emotions",
    "individuals",
    "masks",
    "memories",
    "relationships",
    "sessions",
    "situations",
    "triggers",
    "websocket",
]
